# --------------------------------------------------------
#    mmqgis_grid - Grid creation plugin
#
#    begin                : August 5, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

import os.path
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_grid_form import *

class mmqgis_grid(QDialog, Ui_mmqgis_grid_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		self.crs = QgsCoordinateReferenceSystem()
		self.crs.createFromProj4("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs")
		extent = QgsRectangle (-10.0, -10.0, 10.0, 20.0)

		if (self.iface.mapCanvas() != None) and (self.iface.mapCanvas().mapRenderer() != None):
			self.crs = self.iface.mapCanvas().mapRenderer().destinationSrs()
			extent = self.iface.mapCanvas().mapRenderer().extent()

		elif self.iface.activeLayer() != None:
			extent = self.iface.activeLayer().extent()
			self.crs = self.iface.activeLayer().srs()

		# print self.crs.toProj4()

		spacing = 1
		if extent.width() > 10000:
			spacing = 1000
		elif extent.width() > 1000:
			spacing = 100
		elif extent.width() > 100:
			spacing = 10

		self.originx.setText(str(round(extent.center().x() / spacing) * spacing))
		self.originy.setText(str(round(extent.center().y() / spacing) * spacing))
		self.width.setText(str(round(extent.width() / spacing) * spacing))
		self.height.setText(str(round(extent.height() / spacing) * spacing))
		self.hspacing.setText(str(spacing))
		self.vspacing.setText(str(spacing))

		self.filename.setText(os.getcwd() + "/grid.shp")
		

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def run(self):
		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Grid", "No output filename given")
			return

		hspacing = float(self.hspacing.displayText())
		vspacing = float(self.vspacing.displayText())
		if (hspacing <= 0) or (vspacing <= 0):
			QMessageBox.information(self.iface.mainWindow(), "Grid", "Invalid grid spacing: "
				+ str(hspacing) + " / " + str(vspacing))
			return
		
		width = float(self.width.displayText())
		height = float(self.height.displayText())
		if (width <= hspacing) or (width < vspacing):
			QMessageBox.information(self.iface.mainWindow(), "Grid", "Invalid width / height: "
				+ str(width) + " / " + str(height))
			return
			
		originx = float(self.originx.displayText())
		originy = float(self.originy.displayText())

		fields = {
			0 : QgsField("LINETYPE", QVariant.String),
			1 : QgsField("LABEL", QVariant.Double, "real", 24, 16, "Lat / Long") }

		if QFile(savename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(savename)):
				QMessageBox.information(self.iface.mainWindow(), "Grid", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(savename), QString("System"), fields, QGis.WKBLineString, self.crs);
		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Grid", 
				"Failure creating output shapefile:" + str(outfile.hasError()));
			return
			
		linecount = 0
		x = originx - (width / 2)
		while x < (originx + (width / 2)):
			polyline = []
			geometry = QgsGeometry()
			feature = QgsFeature()
			
			y = originy - (height / 2)
			while y < (originy + (height / 2)):
				polyline.append(QgsPoint(x, y))
				y = y + vspacing;

			feature.setGeometry(geometry.fromPolyline(polyline))
			feature.addAttribute(0, QVariant("Vertical"))
			feature.addAttribute(1, QVariant(x))
			outfile.addFeature(feature)
			linecount = linecount + 1
			#self.iface.mainWindow().statusBar().showMessage("Line " + str(linecount) + " " + str(x))
			x = x + hspacing;

		y = originy - (height / 2)
		while y < (originy + (height / 2)):
			polyline = []
			geometry = QgsGeometry()
			feature = QgsFeature()
			
			x = originx - (width / 2)
			while x < (originx + (width / 2)):
				polyline.append(QgsPoint(x, y))
				x = x + hspacing;

			feature.setGeometry(geometry.fromPolyline(polyline))
			feature.addAttribute(0, QVariant("Horizontal"))
			feature.addAttribute(1, QVariant(y))
			outfile.addFeature(feature)
			linecount = linecount + 1
			#self.iface.mainWindow().statusBar().showMessage("Line " + str(linecount) + " " + str(y))
			y = y + hspacing;

		del outfile

		if self.addtoproject.isChecked():
			vlayer = self.iface.addVectorLayer(savename, "grid", "ogr")
			
		self.iface.mainWindow().statusBar().showMessage(str(linecount) + " gridline shapefile created")



