# --------------------------------------------------------
#    mmqgis_hub_distance - Create shapefile of distances
#			   from points to nearest hub
#
#    begin                : November 19, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from math import *
from qgis.core import *
from mmqgis_hub_distance_form import *

import os.path
#import operator

class mmqgis_hub:
	def __init__(self, newx, newy, newname):
		self.x = newx
		self.y = newy
		self.name = newname

class mmqgis_hub_distance(QDialog, Ui_mmqgis_hub_distance_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayerbox.addItem(layer.name())
				self.hubslayerbox.addItem(layer.name())

		QObject.connect(self.hubslayerbox, SIGNAL("currentIndexChanged(QString)"), self.set_name_attributes)

		self.set_name_attributes()

		self.outputtype.addItem("Point")
		self.outputtype.addItem("Line to Hub")

		self.filename.setText(os.getcwd() + "/distances.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_name_attributes(self):
		self.nameattributebox.clear()
		layername = self.hubslayerbox.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.nameattributebox.addItem(field.name())

	def run(self):
		# Source layer
		sourcelayer = None
		layername = str(self.sourcelayerbox.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				sourcelayer = searchlayer

		if (sourcelayer == None) or (sourcelayer.featureCount() <= 0):
			QMessageBox.information(self.iface.mainWindow(), "Hub Distance", \
				"Project has no active vector layer of starting points")
			return

		# Destination layer
		hubslayer = None
		layername = str(self.hubslayerbox.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				hubslayer = searchlayer

		if (hubslayer == None) or (hubslayer.featureCount() <= 0):
			QMessageBox.information(self.iface.mainWindow(), "Hub Distance", \
				"Project has no active vector layer of destination hubs")
			return

		# Name attributes
		nameattributename = self.nameattributebox.currentText()
		nameindex = hubslayer.dataProvider().fieldNameIndex(nameattributename)
		if nameindex < 0:
			QMessageBox.information(self.iface.mainWindow(), "Hub Distance", \
				"Invalid name attribute: " + nameattributename)

		outputtype = QGis.WKBPoint
		if self.outputtype.currentText() == "Line to Hub":
			outputtype = QGis.WKBLineString

		# Create output file
		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Hub Distance", "No output filename given")
			return

		if QFile(savename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(savename)):
				QMessageBox.information(self.iface.mainWindow(), "Sort", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfields = sourcelayer.dataProvider().fields()
		outfields[len(outfields)] = QgsField(QString("HubName"), QVariant.String)
		outfields[len(outfields)] = QgsField(QString("HubDist"), QVariant.Double)

                outfile = QgsVectorFileWriter(QString(savename), QString("System"), outfields, \
				outputtype, sourcelayer.dataProvider().crs())

		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Sort", 
				"Failure creating output shapefile:" + str(outfile.hasError()));
			return


		# Create array of hubs
		hubs = []
		feature = QgsFeature()
		hubslayer.dataProvider().select(hubslayer.dataProvider().attributeIndexes())
		hubslayer.dataProvider().rewind()
                while hubslayer.dataProvider().nextFeature(feature):
			self.iface.mainWindow().statusBar().showMessage("Reading hub " + str(feature.id()))
			hub = mmqgis_hub(feature.geometry().boundingBox().center().x(), \
				feature.geometry().boundingBox().center().y(), \
				feature.attributeMap()[nameindex].toString())
			hubs.append(hub)

		del hubslayer


		# Scan source points, find nearest hub, and write to output file
		writecount = 0
		feature = QgsFeature()
		sourcelayer.dataProvider().select(sourcelayer.dataProvider().attributeIndexes())
		sourcelayer.dataProvider().rewind()
                while sourcelayer.dataProvider().nextFeature(feature):
			sourcex = feature.geometry().boundingBox().center().x()
			sourcey = feature.geometry().boundingBox().center().y()
 
			hubx = hubs[0].x
			huby = hubs[0].y
			hubname = hubs[0].name
			hubdist = sqrt(pow(sourcex - hubx, 2.0) + pow(sourcey - huby, 2.0))
			for hub in hubs:
				#print "Scanning hub: " + str(hub.x) + "," + str(hub.y) + " " + hub.name
				thisdist = sqrt(pow(sourcex - hub.x, 2.0) + pow(sourcey - hub.y, 2.0))
				if thisdist < hubdist:
					hubx = hub.x
					huby = hub.y
					hubdist = thisdist
					hubname = hub.name

			attributes = feature.attributeMap()
			attributes[len(attributes)] = QVariant(hubname)
			attributes[len(attributes)] = QVariant(hubdist)

			outfeature = QgsFeature()
			outfeature.setAttributeMap(attributes)

			if outputtype == QGis.WKBPoint:
				geometry = QgsGeometry()
				outfeature.setGeometry(geometry.fromPoint(QgsPoint(sourcex, sourcey)))
			else:
				polyline = []
				polyline.append(QgsPoint(sourcex, sourcey))
				polyline.append(QgsPoint(hubx, huby))
				geometry = QgsGeometry()
				outfeature.setGeometry(geometry.fromPolyline(polyline))

			outfile.addFeature(outfeature)

			writecount += 1
			self.iface.mainWindow().statusBar().showMessage("Writing feature " + str(writecount) +\
				" of " + str(sourcelayer.dataProvider().featureCount()))

		del outfile

		if self.addtoproject.isChecked():
			vlayer = self.iface.addVectorLayer(savename, "hubdistance", "ogr")
			
		self.iface.mainWindow().statusBar().showMessage("Hub distance file created from " + \
			self.sourcelayerbox.currentText())




