# ----------------------------------------------------------
#    mmqgis_select - Select features by attribute
#
#    begin                : 2 January 2010
#    copyright            : (c) 2010 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# ----------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_select_form import *

import os.path
import operator

class mmqgis_select(QDialog, Ui_mmqgis_select_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_select_attributes)
		self.set_select_attributes()

		comparisons = ['==', '!=', '>', '>=', '<', '<=', 'begins with', 'contains']
		for x in comparisons:
			self.comparison.addItem(x)

		self.filename.setText(os.getcwd() + "/temp.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_select_attributes(self):
		self.selectattribute.clear()
		layername = self.sourcelayer.currentText()
		for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if selectlayer.name() == layername:
				for index, field in selectlayer.dataProvider().fields().iteritems():
					self.selectattribute.addItem(field.name())

	def run(self):
		layer = None
		layername = str(self.sourcelayer.currentText())
		for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if selectlayer.name() == layername:
				layer = selectlayer

		if layer == None:
			QMessageBox.information(self.iface.mainWindow(), "Select Layer", \
				"Project has no active vector layer to select from")
			return

		selectattributename = self.selectattribute.currentText()
		selectindex = layer.dataProvider().fieldNameIndex(selectattributename)
		if selectindex < 0:
			QMessageBox.information(self.iface.mainWindow(), "Select", \
				"Invalid select field name: " + selectattributename)
			return
		print  "selectindex = " + str(selectindex)

		comparisonname = self.comparison.currentText()
		comparisonindex = self.comparison.currentIndex()
		comparisonvalue = str(self.value.displayText()).strip()
		if comparisonvalue == "":
			QMessageBox.critical(self.iface.mainWindow(), "Select", "No comparison value given")
			return

		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.critical(self.iface.mainWindow(), "Select", "No output filename given")
			return

		if QFile(savename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(savename)):
				QMessageBox.information(self.iface.mainWindow(), "Select", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(savename), QString("System"), layer.dataProvider().fields(),
				layer.dataProvider().geometryType(), layer.dataProvider().crs())

		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Select", 
				"Failure creating output shapefile:" + str(outfile.hasError()));
			return

		readcount = 0
		writecount = 0
		feature = QgsFeature()
		layer.dataProvider().select(layer.dataProvider().attributeIndexes())
		layer.dataProvider().rewind()
                while layer.dataProvider().nextFeature(feature):
			if (comparisonindex >= 6) or (feature.attributeMap()[selectindex].type() == QVariant.String):
				x = str(feature.attributeMap()[selectindex].toString())
				y = comparisonvalue
			else:
				# print feature.attributeMap()[selectindex].typeName()
				x = float(feature.attributeMap()[selectindex].toString())
				y = float(comparisonvalue)

			match = False
			if (comparisonindex == 0):
				match = (x == y)
			elif (comparisonindex == 1):
				match = (x != y)
			elif (comparisonindex == 2):
				match = (x > y)
			elif (comparisonindex == 3):
				match = (x >= y)
			elif (comparisonindex == 4):
				match = (x < y)
			elif (comparisonindex == 5):
				match = (x <= y)
			elif (comparisonindex == 6):
				match = x.startswith(y)
			elif (comparisonindex == 7):
				match = (x.find(y) >= 0)
		
			readcount += 1
			if (match):
				outfile.addFeature(feature)
				writecount += 1

			self.iface.mainWindow().statusBar().showMessage("Scanning feature " + \
				str(readcount) + " of " + str(layer.dataProvider().featureCount()) + \
				"(" + str(writecount) + " selected)")

		del outfile

		if self.addtoproject.isChecked():
			vlayer = self.iface.addVectorLayer(savename, savename, "ogr")
			
		self.iface.mainWindow().statusBar().showMessage("Selected " + str(writecount) + " features to " + savename)



