# --------------------------------------------------------
#    mmqgis_sort - Sort shapefile by attribute
#
#    begin                : August 9, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_sort_form import *

import os.path
import operator

class mmqgis_sort(QDialog, Ui_mmqgis_sort_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())
		#QObject.connect(self.sourcelayer, SIGNAL("editTextChanged(QString)"), self.set_sort_attributes)
		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_sort_attributes)

		self.set_sort_attributes()

		self.filename.setText(os.getcwd() + "/sorted.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_sort_attributes(self):
		self.sortattribute.clear()
		layername = self.sourcelayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.sortattribute.addItem(field.name())

	def run(self):
		layer = None
		layername = str(self.sourcelayer.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				layer = searchlayer

		if layer == None:
			QMessageBox.information(self.iface.mainWindow(), "Sort Layer", \
				"Project has no active vector layer to sort")
			return

		#sortindex = -1
		#sortattributename = self.sortattribute.currentText()
		#for index, fieldname in layer.dataProvider().fields().iteritems():
		#	if fieldname.name() == sortattributename:
		#		sortindex = index

		sortattributename = self.sortattribute.currentText()
		sortindex = layer.dataProvider().fieldNameIndex(sortattributename)
		if sortindex < 0:
			QMessageBox.information(self.iface.mainWindow(), "Sort", \
				"Invalid sort field name: " + sortattributename)
		print  "sortindex = " + str(sortindex)

		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Sort", "No output filename given")
			return

		if QFile(savename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(savename)):
				QMessageBox.information(self.iface.mainWindow(), "Sort", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(savename), QString("System"), layer.dataProvider().fields(),
				layer.dataProvider().geometryType(), layer.dataProvider().crs())

		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Sort", 
				"Failure creating output shapefile:" + str(outfile.hasError()));
			return

		table = []
		feature = QgsFeature()
		layer.dataProvider().select(layer.dataProvider().attributeIndexes())
		layer.dataProvider().rewind()
                while layer.dataProvider().nextFeature(feature):
			# Have to fetch by ID because nextFeature doesn't always seem to get the attributed
			# layer.featureAtId(feature.id(), feature)
			record = feature.id(), str(feature.attributeMap()[sortindex].toString())
			self.iface.mainWindow().statusBar().showMessage("Reading feature " + str(feature.id()))
			table.append(record)

		table.sort(key = operator.itemgetter(1))

		writecount = 0
		for record in table:
			feature = QgsFeature()
			layer.featureAtId(record[0], feature)
			outfile.addFeature(feature)
			writecount += 1
			self.iface.mainWindow().statusBar().showMessage("Writing feature " + str(writecount) +\
				" of " + str(len(table)))

		del outfile

		if self.addtoproject.isChecked():
			vlayer = self.iface.addVectorLayer(savename, "sorted", "ogr")
			
		self.iface.mainWindow().statusBar().showMessage("Sorted shapefile created from " + layername)




