# --------------------------------------------------------
#    mmqgis_street_geocode - Geocode addresses from street 
#			     address finder shapefile
#
#    begin                : August 14, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_street_geocode_form import *
from math import *

import csv
#import math
import os.path
import operator

class mmqgis_street_geocode(QDialog, Ui_mmqgis_street_geocode_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse_infile, SIGNAL("clicked()"), self.browse_infile_dialog)
		QObject.connect(self.browse_shapefile, SIGNAL("clicked()"), self.browse_shapefile_dialog)
		QObject.connect(self.browse_notfound, SIGNAL("clicked()"), self.browse_notfound_dialog)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.streetlayer.addItem(layer.name())

		QObject.connect(self.streetlayer, SIGNAL("currentIndexChanged(int)"), self.set_layer_attributes)
		#QObject.connect(self.streetlayer, SIGNAL("activated(int)"), self.set_layer_attributes)

		self.set_layer_attributes(0)

		self.shapefilename.setText(os.getcwd() + "/geocoded.shp")
		self.notfoundfilename.setText(os.getcwd() + "/notfound.csv")

        def browse_infile_dialog(self):
		newname = QFileDialog.getOpenFileName(None, QString.fromLocal8Bit("Address CSV Input File"), 
			self.infilename.displayText(), "*.csv")

                if newname:
			try:
				infile = open(newname, 'r')
			except:
				QMessageBox.information(self.iface.mainWindow(), "CSV File", "Failure opening " + newname)
				return
			dialect = csv.Sniffer().sniff(infile.read(1024))
			infile.seek(0)
			reader = csv.reader(infile, dialect)
			header = reader.next()
			del reader

			address_index = -1
			self.addressfield.clear()
			for field in header:
				self.addressfield.addItem(QString(field))
				if field.strip().lower() == "address":
					address_index = self.addressfield.count() - 1

			if address_index >= 0:
				self.addressfield.setCurrentIndex(address_index)

                	self.infilename.setText(QString(newname))

        def browse_notfound_dialog(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Not Found List Output File"), 
			self.notfoundfilename.displayText(), "*.csv")
                if newname != None:
                	self.notfoundfilename.setText(QString(newname))

        def browse_shapefile_dialog(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.shapefilename.displayText(), "*.shp")
                if newname != None:
                	self.shapefilename.setText(QString(newname))

	def set_layer_attributes(self, index):
		layer = None
		layername = str(self.streetlayer.currentText())
		for name, streetlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if streetlayer.name() == layername:
				layer = streetlayer

		if not layer:
			print "Layer not found " + layername
			return

		if (layer.dataProvider().crs().mapUnits() == QGis.Feet):
                	self.setback.setText("60")
		elif (layer.dataProvider().crs().mapUnits() == QGis.Meters):
                	self.setback.setText("4")
		elif (layer.dataProvider().crs().mapUnits() == QGis.Degrees):
			meters = 4 * cos(layer.extent().center().y() * pi / 180) * 6378137.0
                	self.setback.setText(str(meters))
		else:
                	self.setback.setText("4")

		self.streetname.clear()
		self.fromx.clear()
		self.fromy.clear()
		self.tox.clear()
		self.toy.clear()
		self.leftfrom.clear()
		self.leftto.clear()
		self.rightfrom.clear()
		self.rightto.clear()

		for index, field in layer.dataProvider().fields().iteritems():
			self.streetname.addItem(str(field.name()))
			self.fromx.addItem(str(field.name()))
			self.fromy.addItem(str(field.name()))
			self.tox.addItem(str(field.name()))
			self.toy.addItem(str(field.name()))
			self.leftfrom.addItem(str(field.name()))
			self.leftto.addItem(str(field.name()))
			self.rightfrom.addItem(str(field.name()))
			self.rightto.addItem(str(field.name()))

		for index, field in layer.dataProvider().fields().iteritems():
			if str(field.name()).lower() == "street":
				self.streetname.setCurrentIndex(index)
			elif str(field.name()).lower() == "xfrom":
				self.fromx.setCurrentIndex(index)
			elif str(field.name()).lower() == "yfrom":
				self.fromy.setCurrentIndex(index)
			elif str(field.name()).lower() == "xto":
				self.tox.setCurrentIndex(index)
			elif str(field.name()).lower() == "yto":
				self.toy.setCurrentIndex(index)
			elif str(field.name()).lower() == "fromleft":
				self.leftfrom.setCurrentIndex(index)
			elif str(field.name()).lower() == "fromright":
				self.rightfrom.setCurrentIndex(index)
			elif str(field.name()).lower() == "toleft":
				self.leftto.setCurrentIndex(index)
			elif str(field.name()).lower() == "toright":
				self.rightto.setCurrentIndex(index)

	def searchable_name(self, name):
		# Use common address abbreviations to reduce naming discrepancies and improve hit ratio
		# print "searchable_name(" + str(name) + ")"
		if not name:
			return ""
		name = name.strip().lower()
		name = name.replace(".", "")
		name = name.replace(" street", " st")
		name = name.replace(" avenue", " av")
		name = name.replace(" plaza", " plz")
		name = name.replace(" ave", " av")
		name = name.replace("saint ", "st ")
		name = name.replace("fort ", "ft ")

		name = name.replace("east", "e")
		name = name.replace("west", "w")
		name = name.replace("north", "n")
		name = name.replace("south", "s")
		name = name.replace("1st", "1")
		name = name.replace("2nd", "2")
		name = name.replace("3rd", "3")
		name = name.replace("4th", "4")
		name = name.replace("5th", "5")
		name = name.replace("6th", "6")
		name = name.replace("7th", "7")
		name = name.replace("8th", "8")
		name = name.replace("9th", "9")
		name = name.replace("0th", "0")
		name = name.replace("1th", "1")
		name = name.replace("2th", "2")
		name = name.replace("3th", "3")
		return name

	def run(self):
		layer = None
		layername = str(self.streetlayer.currentText())
		for name, streetlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if streetlayer.name() == layername:
				layer = streetlayer

		if layer == None:
			QMessageBox.information(self.iface.mainWindow(), "Address Geocode", "Address layer not found")
			return

		if len(self.infilename.displayText()) <= 0:
			QMessageBox.information(self.iface.mainWindow(), "Address Geocode", "No input CSV file given")
			return

		# Read the CSV file data into memory
		try:
			infile = open(self.infilename.displayText(), 'r')
		except:
			QMessageBox.information(self.iface.mainWindow(), "CSV File", "Failure opening " + 
				self.infilename.displayText())
			return

		dialect = csv.Sniffer().sniff(infile.read(1024))
		infile.seek(0)
		reader = csv.reader(infile, dialect)

		fields = {}
		header = reader.next()
		for field in header:
			fields[len(fields)] = QgsField(field[0:9].strip(), QVariant.String)

		longfield = len(fields)
		latfield = longfield + 1
		fields[longfield] = QgsField("Longitude", QVariant.Double, "real", 24, 16)
		fields[latfield] = QgsField("Latitude", QVariant.Double, "real", 24, 16)

		csv_found = []
		csv_attributes = []
		csv_streetname = []
		csv_streetnumber = []
		addressfield_index = self.addressfield.currentIndex();
		for row in reader:
			address = self.searchable_name(row[addressfield_index])
			x = 0
			while (x < len(address)) and (address[x].isdigit() or (address[x] == "-")):
				x += 1

			if (x > 0):
				number = int(address[0:x].replace("-", ""))
			else:
				number = 0
			csv_streetnumber.append(number)

			street = address[x:].strip().lower()
			csv_streetname.append(street)

			# print str(x) + " \"" + address + "\", " + str(number) + ", " + street

			attributes = {}
			for field in row:
				attributes[len(attributes)] = QVariant(field)
				
			csv_attributes.append(attributes)
			csv_found.append(0)

		del reader
		del infile

		# Create the output shapefile
		shapefilename = self.shapefilename.displayText()
		if QFile(shapefilename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(shapefilename)):
				QMessageBox.information(self.iface.mainWindow(), "Address Geocode", \
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(shapefilename), QString("System"), \
				fields, QGis.WKBPoint, layer.dataProvider().crs())
		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Address Geocode", \
				"Failure creating output shapefile:" + str(outfile.hasError()));
			return


		streetname_attribute = self.streetname.currentIndex()
		fromx_attribute = self.fromx.currentIndex()
		fromy_attribute = self.fromy.currentIndex()
		tox_attribute = self.tox.currentIndex()
		toy_attribute = self.toy.currentIndex()
		leftfrom_attribute = self.leftfrom.currentIndex()
		rightfrom_attribute = self.rightfrom.currentIndex()
		leftto_attribute = self.leftto.currentIndex()
		rightto_attribute = self.rightto.currentIndex()
                setback = float(self.setback.displayText())

		# Iterate through each feature in the source layer
		matched_count = 0
		feature_count = layer.dataProvider().featureCount()

		feature = QgsFeature()
		layer.dataProvider().select(layer.dataProvider().attributeIndexes())
		layer.dataProvider().rewind()
                while layer.dataProvider().nextFeature(feature):
			if (feature.id() % 20) == 0:
				self.iface.mainWindow().statusBar().showMessage("Searching street " + \
					str(feature.id()) + " of " + str(feature_count) + \
					" (" + str(matched_count) + " matched)")

			attributes = feature.attributeMap()
			key = self.searchable_name(str(attributes[streetname_attribute].toString()))

			# Check each address against this feature
			for row in range(0, len(csv_attributes)):
				if (not csv_found[row]) and (csv_streetname[row] == key):
					(leftto, test) = attributes[leftto_attribute].toInt()
					(leftfrom, test) = attributes[leftfrom_attribute].toInt()
					(rightto, test) = attributes[rightto_attribute].toInt()
					(rightfrom, test) = attributes[rightfrom_attribute].toInt()
					# print "Street found \"" + key + "\", " + str(leftfrom) + ", " + str(leftto)

					if ((csv_streetnumber[row] >= leftfrom) and (csv_streetnumber[row] <= leftto) \
					    and ((leftfrom % 2) == (csv_streetnumber[row] % 2))) or \
					   ((csv_streetnumber[row] >= rightfrom) and (csv_streetnumber[row] <= rightto) \
					    and ((rightfrom % 2) == (csv_streetnumber[row] % 2))):
						left = ((leftfrom % 2) == (csv_streetnumber[row] % 2))
						if left:
							if (leftfrom == leftto):
								ratio = 0.5
							else:
								ratio = float(csv_streetnumber[row] - leftfrom) \
									/ float(leftto - leftfrom)
						else:
							if (rightfrom == rightto):
								ratio = 0.5
							else:
								ratio = float(csv_streetnumber[row] - rightfrom) \
									/ float(rightto - rightfrom)
					
						(tox, test) = attributes[tox_attribute].toDouble()
						(toy, test) = attributes[toy_attribute].toDouble()
						(fromx, test) = attributes[fromx_attribute].toDouble()
						(fromy, test) = attributes[fromy_attribute].toDouble()

						# setback from corner
						angle = atan2(toy - fromy, tox - fromx)
						setback_fromx = fromx + (setback * cos(angle))
						setback_tox = tox - (setback * cos(angle))
						setback_fromy = fromy + (setback * sin(angle))
						setback_toy = toy - (setback * sin(angle))

						x = setback_fromx + ((setback_tox - setback_fromx) * ratio)
						y = setback_fromy + ((setback_toy - setback_fromy) * ratio)

						# setback from street center
						if left:
							y += (setback * cos(angle))
							x -= (setback * sin(angle))
						else:
							y -= (setback * cos(angle))
							x += (setback * sin(angle))

						csv_attributes[row][longfield] = QVariant(x)
						csv_attributes[row][latfield] = QVariant(y)

						newfeature = QgsFeature()
						newfeature.setAttributeMap(csv_attributes[row])
						geometry = QgsGeometry.fromPoint(QgsPoint(x, y))
						newfeature.setGeometry(geometry)
						outfile.addFeature(newfeature)
						matched_count += 1
						csv_found[row] += 1

		del outfile

		# Write records that were not joined to the notfound file
		try:
			outfile = open(self.notfoundfilename.displayText(), 'w')
		except:
			QMessageBox.information(self.iface.mainWindow(), "CSV File", "Failure opening " + 
				self.notfoundfilename.displayText())
		else:
			writer = csv.writer(outfile, dialect)
			writer.writerow(header)
			for x in range(0, len(csv_attributes)):
				if not csv_found[x]:
					row = []
					for key, value in csv_attributes[x].iteritems():
						row.append(value.toString())
					writer.writerow(row)
			del outfile

		if matched_count and self.addtoproject.isChecked():
			vlayer = self.iface.addVectorLayer(shapefilename, "addresses", "ogr")
			
		self.iface.mainWindow().statusBar().showMessage(str(matched_count) + " of " + str(len(csv_attributes)) \
			+ " addresses geocoded from " + str(feature_count) + " street records")


