# ---------------------------------------------------------
#    mmqgis_text_to_float - Change text fields to numbers
#
#    begin                : September 3, 2009
#    copyright            : (c) 2009 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# ---------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from mmqgis_text_to_float_form import *

import os.path
import operator

class mmqgis_text_to_float(QDialog, Ui_mmqgis_text_to_float_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_fieldnames)

		self.set_fieldnames()

		self.filename.setText(os.getcwd() + "/numeric.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_fieldnames(self):
		self.fieldnames.clear()
		layername = self.sourcelayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.fieldnames.addItem(field.name())

				for index, field in searchlayer.dataProvider().fields().iteritems():
					if (field.type() == QVariant.String):
						self.fieldnames.item(index).setSelected(1)

	def run(self):
		layer = None
		layername = str(self.sourcelayer.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				layer = searchlayer

		if layer == None:
			QMessageBox.information(self.iface.mainWindow(), "Text to Float", \
				"Project has no active vector layer to convert")
			return

		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Text to Float", "No output filename given")
			return

		# Build dictionary of fields with selected fields for conversion to floating point
		destfields = {};
		changecount = 0
		for index, field in layer.dataProvider().fields().iteritems():
			destfields[index] = QgsField (field.name(), field.type(), field.typeName(), \
				field.length(), field.precision(), field.comment())
 
			#print "Scan " + str(destfields[index].name())
			if self.fieldnames.item(index).isSelected():
				#print "Change " + str(destfields[index].name())
				#if (field.type() != QVariant.String) and (field.type() != QVariant.Double) \
				#    and (field.type() != QVariant.Int):
				#	QMessageBox.information(self.iface.mainWindow(), "Text to Float", \
				#		"Field " + str(field.name()) + " is not a string or number (" + \
				#		str(field.type()) + " " + str(field.typeName()) + ")")
				#	return
				if (field.type() == QVariant.String) or (field.type() == QVariant.Int):
					changecount += 1
					destfields[index].setType(QVariant.Double)

		if (changecount <= 0):
			QMessageBox.information(self.iface.mainWindow(), "Text to Float", 
				"No string or integer fields selected for conversion to floating point")
			return


		# Create the output file
		if QFile(savename).exists():
			if not QgsVectorFileWriter.deleteShapeFile(QString(savename)):
				QMessageBox.information(self.iface.mainWindow(), "Text to Float", 
					"Failure deleting existing shapefile: " + savename)
				return

                outfile = QgsVectorFileWriter(QString(savename), QString("System"), destfields,
				layer.dataProvider().geometryType(), layer.dataProvider().crs())

		if (outfile.hasError() != QgsVectorFileWriter.NoError):
			QMessageBox.information(self.iface.mainWindow(), "Text to Float", 
				"Failure creating output shapefile:" + str(outfile.hasError()));
			return



		# Write the features with modified attributes
		feature = QgsFeature()
		featurecount = layer.dataProvider().featureCount();
		layer.dataProvider().select(layer.dataProvider().attributeIndexes())
		layer.dataProvider().rewind()
                while layer.dataProvider().nextFeature(feature):
			self.iface.mainWindow().statusBar().showMessage("Writing feature " + \
				str(feature.id()) + " of " + str(featurecount))

			attributes = feature.attributeMap()
			for index, field in layer.dataProvider().fields().iteritems():
				if (field.type() != destfields[index].type()):
					#print str(index) + ") " + str(destfields[index].typeName()) + \
					#	": " + str(attributes[index].toString())
					string = str(attributes[index].toString())
					multiplier = 1.0
					if (string.find("%") >= 0):
						multiplier = 1 / 100.0
						string = string.replace("%", "")
					try:	
						value = float(string) * multiplier
					except:
						value = 0
							
					attributes[index] = QVariant(value)

			#for index, field in attributes.iteritems():
			#	print "  " + str(index) + " = " + str(field.toString())

			feature.setAttributeMap(attributes)
			outfile.addFeature(feature)

		del outfile

		if self.addtoproject.isChecked():
			vlayer = self.iface.addVectorLayer(savename, "numeric", "ogr")
			
		self.iface.mainWindow().statusBar().showMessage(str(changecount) + " text converted to numeric")




